
#include <reconRootData/ReconEvent.h>
#include <commonRootData/RootDataUtil.h>
#include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TRandom.h"
#include "TCollection.h"
#include "TVector3.h"
#include "Riostream.h"

const UInt_t RUN_NUM = 1;
Float_t RAND_NUM;

int checkReconEvent(ReconEvent *evt, UInt_t ievent) {
    
    if (evt->getRunId() != RUN_NUM) {
        std::cout << "Run Id is wrong: " << evt->getRunId() << std::endl;
        return -1;
    }
    if (evt->getEventId() != ievent) {
        std::cout << "Event Id is wrong: " << evt->getEventId() << std::endl;
        return -1;
    }

    return 0;
    
}

/// Read in the ROOT file just generated via the write method
int read(char* fileName, int numEvents) {
    TFile *f = new TFile(fileName, "READ");
    TTree *t = (TTree*)f->Get("Recon");
    ReconEvent *evt = 0;
    t->SetBranchAddress("ReconEvent", &evt);

    std::cout << "Opened the ROOT file for reading" << std::endl;
    std::cout << "Random number is " << RAND_NUM << std::endl;

    UInt_t ievent;
    for (ievent = 0; ievent < (UInt_t) numEvents; ievent++) {
        
        t->GetEvent(ievent);
        std::cout << "ReconEvent ievent = " << ievent << std::endl;
        evt->Print() ;
        
        if (checkReconEvent(evt, ievent) < 0)
         { return -1 ; }
        
        // prepare data
        AcdRecon * acd = evt->getAcdRecon() ;
//        CalRecon * cal = evt->getCalRecon() ;
//        TkrRecon * tkr = evt->getTkrRecon() ;
//        reconRootData::AdfRecon * adf = evt->getAdfRecon();
//        if ((!acd)||(!cal)||(!tkr)||(!adf))
//         { return -1 ; }
        acd->Print() ;
//        cal->Print();
//        tkr->Print();
//        adf->Print();
        AcdRecon acdRef ;
//        CalRecon calRef ;
//        TkrRecon tkrRef ;
//        reconRootData::AdfRecon adfRef;
         
        // usual tests
        acdRef.Fake(ievent,RAND_NUM) ;
        if (!acd->CompareInRange(acdRef))
         { return -1 ; }
//        calRef.Fake(ievent,RAND_NUM) ;
//        if (!cal->CompareInRange(calRef))
//         { return -1 ; }
//        tkrRef.Fake(ievent,RAND_NUM) ;
//        if (!tkr->CompareInRange(tkrRef))
//         { return -1 ; }
//        adfRef.Fake(ievent,RAND_NUM);
//        if (!adf->CompareInRange(adfRef))
//         { return -1; }
        
        evt->Clear();
    }

    f->Close();
    delete f;

    return 0;
}


/// Main program
/// Return 0 for success.
/// Returns -1 for failure.
int main(int argc, char **argv) {
    
    char *fileName = "recon.root" ;
    int n =1 ;
    unsigned int numEvents =10 ;
    if (argc > 1) {
        fileName = argv[n++] ;
    } 
    if (argc > 2) {
        numEvents = atoi(argv[n++]) ;
    } 

    int sc = 0 ;
    try 
     {
      RAND_NUM = 0.1543 ;
      sc = read(fileName,numEvents) ;
     }
    catch (...)
     {
      std::cout<<"AN UNKNOWN EXCEPTION HAS BEEN RAISED"<<std::endl ;
      sc = 1 ;
     }
         
    if (sc == 0) {
        std::cout << "RECON ROOT file reading succeeded!" << std::endl;
    } else {
        std::cout << "FAILED recon reading" << std::endl;
    }

    return(sc);
}
