
#include <reconRootData/ReconEvent.h>
#include <commonRootData/RootDataUtil.h>
#include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TRandom.h"
#include "TCollection.h"
#include "TVector3.h"
#include "Riostream.h"

const UInt_t RUN_NUM = 1;
Float_t RAND_NUM;

int write(char* fileName, int numEvents) {
    
    Int_t buffer = 64000 ;
    Int_t splitLevel = 1 ;

    TFile * f =  new TFile(fileName,"RECREATE") ;
    TTree * t = new TTree("Recon","Recon") ;
    ReconEvent * ev = new ReconEvent() ;
    t->Branch("ReconEvent", "ReconEvent", &ev, buffer, splitLevel);
    std::cout << "Created new ROOT file" << std::endl;

    std::cout << "Random number is " << RAND_NUM << std::endl;
    
    Int_t ievent ;
    for (ievent = 0; ievent < numEvents; ievent++) {

        AcdRecon * acdRec = new AcdRecon() ;
        acdRec->Fake(ievent,RAND_NUM) ;
        
//        CalRecon * calRec = new CalRecon() ;
//        calRec->Fake(ievent,RAND_NUM) ;
//
//        TkrRecon * tkrRec = new TkrRecon();
//        tkrRec->Fake(ievent,RAND_NUM) ;        
//
//        reconRootData::AdfRecon * adfRec = new reconRootData::AdfRecon();
//        adfRec->Fake(ievent,RAND_NUM);
//
        //ev->initialize(ievent, RUN_NUM, tkrRec, calRec, acdRec);
        ev->initialize(ievent, RUN_NUM, acdRec);
        //ev->initAdf(adfRec);
        ev->initEventFlags(1) ;
        
        t->Fill();
        ev->Clear();
    }

    std::cout << "Filled ROOT file with " << numEvents << " events" << std::endl;
    delete ev;

    f->Write();
    f->Close();
    delete f;
    return(0);
}


/// Main program
/// Return 0 for success.
/// Returns -1 for failure.
int main(int argc, char **argv) {
    
    char *fileName = "recon.root" ;
    int n =1 ;
    unsigned int numEvents =10 ;
    if (argc > 1) {
        fileName = argv[n++] ;
    } 
    if (argc > 2) {
        numEvents = atoi(argv[n++]) ;
    } 

    int sc = 0 ;
    try 
     {
      RAND_NUM = 0.1543 ;
      sc = write(fileName,numEvents) ;
     }
    catch (...)
     {
      std::cout<<"AN UNKNOWN EXCEPTION HAS BEEN RAISED"<<std::endl ;
      sc = 1 ;
     }
         
    if (sc == 0) {
        std::cout << "RECON ROOT file writing succeeded!" << std::endl;
    } else {
        std::cout << "FAILED recon writing" << std::endl;
    }

    return(sc);
}
